Ext.namespace('dSS', 'dSS.data');

dSS.data.ModulatorStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for zone records
        var modulatorRecord = Ext.data.Record.create([
        {
            name: "busId",
            mapping: "busID"
        },
        {
            name: "deviceType"
        },
        {
            name: "dsid"
        },
        {
            name: "hwName"
        },
        {
            name: "hwVersion"
        },
        {
            name: "armSwVersion"
        },
        {
            name: "dspSwVersion"
        },
        {
            name: "apiVersion"
        },
        {
            name: "isPresent"
        },
        {
            name: "name"
        },
        {
            name: "consumption"
        },
        {
            name: "measureDate"
        },
        {
            name: "isValid"
        },
        {
            name: "numDevices",
            defaultValue: 0
        },
        {
            name: "ledCode",
            defaultValue: -1
        },
        {
            name: "ledStatus"
        }
        ]);

        // a json reader to read the zone data
        var modulatorReader = new Ext.data.JsonReader({
            root: "result.circuits",
            idProperty: 'dsid'
        },
        modulatorRecord);

        this.addEvents('loadcomplete');

        Ext.apply(this, {
            reader: modulatorReader,
            url: '/json/apartment/getCircuits'
        });
        dSS.data.ModulatorStore.superclass.constructor.call(this, arguments);

        this.on('exception', function() {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not load data from dSS!"));

                dSS.MessageBus.fireEvent('structureloaded');
            },
            this);
        this.on('load', function(store, records, options) {
            Ext.each(records, function(record) {
                    record.set('hwVersion', dSS.util.decode32(
                                                     record.get('hwVersion')));

                    record.set('armSwVersion', dSS.util.decode32(
                                                  record.get('armSwVersion')));

                    record.set('dspSwVersion', dSS.util.decode32(
                                                  record.get('dspSwVersion')));

                    record.set('apiVersion', dSS.util.decode16V(
                                                  record.get('apiVersion')));
                }
            );

            Ext.Ajax.request({
                url: '/json/metering/getLatest',
                disableCaching: true,
                method: 'GET',
                scope: this,
                params: {
                    type: 'consumption',
                    from: '.meters(all)'
                },
                success: function(result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (jsonData.ok) {
                            var modified = false;

                            Ext.each(jsonData.result.values, function(val) {
                                var index = store.find("dsid", new RegExp("^" + val.dsid + "$"));
                                if (index > - 1) {
                                    var rec = store.getAt(index);
                                    rec.set("consumption", val.value);
                                    rec.set("measureDate", val.date);
                                    modified = true;
                                }
                            });
                            if (modified === true) {
                                store.commitChanges();
                            }

                            this.getARMImageVersion(store);
                        } else {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not get metering data from server"));
                            dSS.MessageBus.fireEvent('structureloaded');
                        }
                    }
                    catch(err) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not get metering data from server"));
                        dSS.MessageBus.fireEvent('structureloaded');
                    }
                },
                failure: function(result, request) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not get metering data from server"));
                    dSS.MessageBus.fireEvent('structureloaded');
                }
            });
        },
        this);
    },
    // have to use two calls because query behaved buggy and delivered
    // non-determenistic results
    getARMImageVersion: function(store) {
        Ext.Ajax.request({
            url: '/json/property/getString',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path : '/system/version/dSMFirmwareImageARM'
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok === true) {
                        if (jsonData.result.value.length > 0) {
                            dSS.config.dSMSystemFirmwareImageVersion.ARM =
                                jsonData.result.value;
                        }
                        this.getDSPImageVersion(store);
                    } else {
                        this.getLedInfo(store);
                    }
                } catch(err) {
                    this.getLedInfo(store);
                }
            },
            failure: function(result, request) {
                this.getLedInfo(store);
            }
        });
    },
    getDSPImageVersion: function(store) {
        Ext.Ajax.request({
            url: '/json/property/getString',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path : '/system/version/dSMFirmwareImageDSP'
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok === true) {
                        if (jsonData.result.value.length > 0) {
                            dSS.config.dSMSystemFirmwareImageVersion.DSP =
                                jsonData.result.value;
                        }
                    }
                } catch(err) { }
                this.getLedInfo(store);
            },
            failure: function(result, request) {
                this.getLedInfo(store);
            }
        });
    },
    getLedInfo: function(store) {
        if (dSA.module !== true) {
            this.fireEvent('loadcomplete');
            return;
        }

        var f = function(success, data) {
            if (success === false) {
                dSS.util.updateDSMLEDInfo(store);
            } else {
                dSS.util.updateDSMLEDInfo(store, data.dsms);
            }
            this.fireEvent('loadcomplete');
        }.createDelegate(this);

        dSA.functions.GetDSMLED("", f, true);
    }
});

